@@ -1,4 +1,4 @@
-#!/usr/local/bin/python3.9
+#!/usr/local/bin/python3.12
 ###########################################################################
 #
 # xasyFile implements the loading, parsing, and saving of an asy file.
@@ -103,15 +103,27 @@ def xasyToDict(file, xasyItems, asy2psmap):
                     })
 
         elif isinstance(item, xasy2asy.xasyShape):
-            penData = {'color': item.pen.color, 'width': item.pen.width, 'options': item.pen.options}
+            penData = {'color': item.pen.color, 'width': item.pen.width, 'dashPattern': item.pen.dashPattern, 'options': item.pen.options}
             fileItems.append({'type': 'xasyShape',
                     'nodes': item.path.nodeSet,
                     'links': item.path.linkSet,
+                    'fill': item.path.fill,
                     'transform': item.transfKeymap[item.transfKey][0].t,
                     'transfKey': item.transfKey,
                     'pen': penData
                     })
 
+        elif isinstance(item, xasy2asy.asyArrow): #Will this ever even be reached?
+            penData = {'color': item.pen.color, 'width': item.pen.width, 'dashPattern': item.pen.dashPattern, 'options': item.pen.options}
+            fileItems.append({'type': 'asyArrow',
+                    'pen': penData,
+                    'arrowSettings': item.arrowSettings,
+                    'transform': item.transfKeymap[item.transfKey][0].t,
+                    'transfKey': item.transfKey,
+                    'settings': item.arrowSettings,
+                    'code': item.code
+                    })
+
         else:
             # DEBUGGING PURPOSES ONLY
             print(type(item))
